#!/usr/bin/python3
#
# (c) Sven Sager, License: GPLv3
#
# -*- coding: utf-8 -*-
import Ui_revolutionsumbau
from PyQt5 import QtWidgets, QtCore, QtGui
from time import strftime
import sys

sys.path.append("/home/akira/Entwicklung/eric/modules/revpimodio2")
import revpimodio2


class MyApp(QtWidgets.QDialog):

    def __init__(self, parent=None):
        super().__init__(parent)
        self.ui = Ui_revolutionsumbau.Ui_Dialog()
        self.ui.setupUi(self)

        self.rpi = None
        self.virt = None

        self.dict_meldungen = {
            "err_offline": ["Revolution Pi OFFLINE", None, False, (255, 0, 0)],
            "info_online": ["Revolution Pi ONLINE", None, None, (255, 255, 255)],
        }

        # Merker
        self.mrk_error1 = False
        self.mrk_timer1 = False
        self.mrk_timer2 = False

        # Timer erzeugen
        self.tmr = QtCore.QTimer(self)
        self.tmr.setSingleShot(True)
        self.tmr.start(1000)
        self.tmr.timeout.connect(self.evt_tmr)

    def _config_rpi(self):
        # RevPiModIOs Instantieren
        self.rpi = revpimodio2.RevPiNetIOSelected(
            "172.29.255.200",
            [32, 33, 34, 35, 36],
            monitoring=True,
            autorefresh=True
        )
        self.virt = revpimodio2.RevPiNetIODriver(
            "172.29.255.200",
            "virt01",
            autorefresh=True,
            syncoutputs=True
        )
        self.virt.net_setdefaultvalues()

        # Panel Eingänge Byte 0
        self.virt.io.meldung0_7.replace_io(
            "err_master", "?", bit=0, event=self.evt_trigger_meldungen)
        self.virt.io.meldung0_7.replace_io(
            "err_funichtaus", "?", bit=1, event=self.evt_trigger_meldungen)
        self.virt.io.meldung0_7.replace_io(
            "err_magazin1", "?", bit=2, event=self.evt_trigger_meldungen)
        self.virt.io.meldung0_7.replace_io(
            "err_magazin2", "?", bit=3, event=self.evt_trigger_meldungen)
        self.virt.io.meldung0_7.replace_io(
            "err_rutsche", "?", bit=4, event=self.evt_trigger_meldungen)
        self.virt.io.meldung0_7.replace_io(
            "err_endlage", "?", bit=5, event=self.evt_trigger_meldungen)

        # Panel Eingänge Byte 1
        self.virt.io.meldung8_15.replace_io(
            "warn_mag1full", "?", bit=0, event=self.evt_trigger_meldungen)
        self.virt.io.meldung8_15.replace_io(
            "warn_mag2full", "?", bit=1, event=self.evt_trigger_meldungen)
        self.virt.io.meldung8_15.replace_io(
            "aktivauto", "?", bit=6, event=self.evt_modus)
        self.virt.io.meldung8_15.replace_io(
            "aktivhand", "?", bit=7, event=self.evt_modus)

        # Panel Ausgänge Byte 0
        self.virt.io.pbit0_7.replace_io("p_online", "?", bit=0)
        self.virt.io.pbit0_7.replace_io("p_mberry", "?", bit=6)
        self.virt.io.pbit0_7.replace_io("p_schnell", "?", bit=7)

        # Panel Ausgänge Byte 1
        self.virt.io.pbit8_15.replace_io("t_m1", "?", bit=0)
        self.virt.io.pbit8_15.replace_io("t_m2", "?", bit=1)
        self.virt.io.pbit8_15.replace_io("t_m3", "?", bit=2)
        self.virt.io.pbit8_15.replace_io("t_rechts", "?", bit=3)
        self.virt.io.pbit8_15.replace_io("t_motorstart", "?", bit=5)
        self.virt.io.pbit8_15.replace_io("t_magazin1", "?", bit=6)
        self.virt.io.pbit8_15.replace_io("t_magazin2", "?", bit=7)

        # Panel Ausgang Byte 7 + 8
        self.virt.io.p_drehzahl1.replace_io("p_drehzahl", "H")

        # RevPi Events registrieren und überwachen
        self.rpi.io.notaus_ok.reg_event(self.evt_trigger_meldungen)
        self.rpi.io.motorschutz_ok.reg_event(self.evt_trigger_meldungen)
        self.rpi.io.fu_ok.reg_event(self.evt_trigger_meldungen)
        self.rpi.io.m_druck_ok.reg_event(self.evt_trigger_meldungen, delay=2000)
        self.rpi.io.mb_status.reg_event(self.evt_mberry)

        # Eventhandling starten
        self.rpi.mainloop(blocking=False)
        self.virt.mainloop(blocking=False)

        # Panel online melden
        self.virt.io.p_online.value = True

    def _config_ui(self):
        # Meldungen beim Start prüfen
        self.dict_meldungen.update({
            "notaus_ok": ["Not-Aus betätigt", self.rpi, False, (255, 0, 0)],
            "m_druck_ok": ["Kein Druck", self.rpi, False, (255, 0, 0)],
            "fu_ok": ["FU Fehler", self.rpi, False, (255, 0, 0)],
            "motorschutz_ok": ["Motorschutz ausgelöst", self.rpi, False, (255, 0, 0)],
            "err_master": ["Master Fehler", self.virt, True, (255, 0, 0)],
            "err_endlage": ["Eine Endlage wurde nicht erreicht", self.virt, True, (255, 0, 0)],
            "err_funichtaus": ["FU schaltet nicht aus", self.virt, True, (255, 0, 0)],
            "err_magazin1": ["Magazin 1 nicht erreicht", self.virt, True, (255, 0, 0)],
            "err_magazin2": ["Magazin 2 nicht erreicht", self.virt, True, (255, 0, 0)],
            "err_rutsche": ["Sensor wird nicht frei", self.virt, True, (255, 0, 0)],
            "aktivauto_an": ["Automatik aktiviert", None, None, (0, 200, 255)],
            "aktivauto_aus": ["Automatik abgeschaltet", None, None, (0, 200, 255)],
            "aktivhand_an": ["Hand aktiviert", None, None, (0, 200, 255)],
            "aktivhand_aus": ["Hand abgeschaltet", None, None, (0, 200, 255)],
            "info_magazin1": ["Magazin 1 geleert", None, None, (255, 255, 255)],
            "info_magazin2": ["Magazin 2 geleert", None, None, (255, 255, 255)],
            "info_stoerreset": ["Störungen beseitigt", None, None, (255, 255, 255)],
            "warn_mag1full": ["Magazin 1 voll", self.virt, True, (255, 255, 0)],
            "warn_mag2full": ["Magazin 2 voll", self.virt, True, (255, 255, 0)]
        })
        for ioname in self.dict_meldungen:
            lst_meldung = self.dict_meldungen[ioname]
            if lst_meldung[1] is not None \
                    and lst_meldung[1].io[ioname].value == lst_meldung[2]:
                self.add_meldung(ioname)

        # UI Signale konfigurieren

        self.ui.btnAutomatik.toggled.connect(
            lambda value: self.evt_btn_chvalue("p_schnell", value))
        self.ui.btnMberry.pressed.connect(
            lambda: self.evt_btn_chvalue("p_mberry", True)
        )
        self.ui.btnMberry.released.connect(
            lambda: self.evt_btn_chvalue("p_mberry", False)
        )

        # Zylindersteuerung
        self.ui.btnHandM1.pressed.connect(
            lambda: self.evt_btn_chvalue("t_m1", True))
        self.ui.btnHandM1.released.connect(
            lambda: self.evt_btn_chvalue("t_m1", False))
        self.ui.btnHandM2.pressed.connect(
            lambda: self.evt_btn_chvalue("t_m2", True))
        self.ui.btnHandM2.released.connect(
            lambda: self.evt_btn_chvalue("t_m2", False))
        self.ui.btnHandM3.pressed.connect(
            lambda: self.evt_btn_chvalue("t_m3", True))
        self.ui.btnHandM3.released.connect(
            lambda: self.evt_btn_chvalue("t_m3", False))

        # Magazinreset
        self.ui.btnResetMag1.pressed.connect(
            lambda: self.evt_btn_resetmagazin("t_magazin1", True))
        self.ui.btnResetMag1.released.connect(
            lambda: self.evt_btn_resetmagazin("t_magazin1", False))
        self.ui.btnResetMag2.pressed.connect(
            lambda: self.evt_btn_resetmagazin("t_magazin2", True))
        self.ui.btnResetMag2.released.connect(
            lambda: self.evt_btn_resetmagazin("t_magazin2", False))

        # Motorsteuerung
        self.ui.btnHandRechts.toggled.connect(
            lambda value: self.evt_btn_chvalue("t_rechts", value))
        self.ui.btnHandStart.pressed.connect(
            lambda: self.evt_btn_chvalue("t_motorstart", True))
        self.ui.btnHandStart.released.connect(
            lambda: self.evt_btn_chvalue("t_motorstart", False))
        self.ui.slDrehzahl.valueChanged.connect(self.evt_setdrehzahl)
        self.ui.slDrehzahl.setValue(self.virt.io.p_drehzahl.value)

        # Meldungsfunktionen
        self.ui.btnClearOne.clicked.connect(self.del_meldung)
        self.ui.btnClearAll.clicked.connect(self.del_meldung_all)
        self.ui.lstMeldung.currentItemChanged.connect(
            self.evt_lstmeldung_currentitemchanged
        )

        # Maxwerte ändern
        self.ui.hsMag1.setValue(self.virt.io.magazin1_max.value)
        self.ui.hsMag2.setValue(self.virt.io.magazin2_max.value)
        self.ui.hsMag1.valueChanged.connect(self.evt_valuechanged)
        self.ui.hsMag2.valueChanged.connect(self.evt_valuechanged)

        # Blinker paletten
        self.po_btnNormal = self.ui.btnResetMag1.palette()
        self.po_btnRed = self.ui.btnResetMag1.palette()
        self.po_btnRed.setColor(
            QtGui.QPalette.Button, QtGui.QColor(255, 0, 0)
        )
        self.po_btnGreen = self.ui.btnResetMag1.palette()
        self.po_btnGreen.setColor(
            QtGui.QPalette.Button, QtGui.QColor(255, 255, 0)
        )

        # UI Werte laden

        # Fernwartung
        if self.rpi.io.mb_status.value:
            self.ui.btnMberry.setAutoFillBackground(True)
            self.ui.btnMberry.setPalette(self.po_btnGreen)
            self.ui.btnMberry.update()

        # Buttonwerte laden
        self.ui.btnAutomatik.setChecked(self.virt.io.p_schnell.value)
        self.ui.btnHandRechts.setChecked(self.virt.io.t_rechts.value)

    def add_meldung(self, ioname):
        lst_meldung = self.dict_meldungen.get(
            ioname, ["Unbekannter Fehler", None, None, (255, 0, 0)]
        )
        item = QtWidgets.QListWidgetItem()
        item.setBackground(QtGui.QBrush(QtGui.QColor(*lst_meldung[3])))
        item.setData(256, QtCore.QVariant(ioname))
        item.setText("{}: {}".format(
            strftime("%Y-%m-%d %H:%M"), lst_meldung[0]
        ))
        self.ui.lstMeldung.insertItem(0, item)

    def del_meldung(self):
        iw = self.ui.lstMeldung.currentItem()
        ioname = iw.data(256)
        lst_meldung = self.dict_meldungen[ioname]
        if lst_meldung[1] is None \
                or lst_meldung[1].io[ioname].value != lst_meldung[2]:
            self.ui.lstMeldung.takeItem(self.ui.lstMeldung.currentRow())
        else:
            QtWidgets.QMessageBox.warning(
                self, "Fehler...",
                "Die Meldung konnte nicht gelöscht werden, da sie noch "
                "ansteht.\nPrüfen Sie den Status der Anlage."
            )

    def del_meldung_all(self):
        bool_err = False
        lst_del = []
        for i in range(self.ui.lstMeldung.count() - 1, -1, -1):
            iw = self.ui.lstMeldung.item(i)
            ioname = iw.data(256)
            lst_meldung = self.dict_meldungen[ioname]
            if lst_meldung[1] is None \
                    or lst_meldung[1].io[ioname].value != lst_meldung[2]:
                lst_del.append(i)
            else:
                bool_err = True

        for delindex in lst_del:
            self.ui.lstMeldung.takeItem(delindex)

        if bool_err:
            QtWidgets.QMessageBox.warning(
                self, "Fehler...",
                "Einige Meldungen konnten nicht gelöscht werden, da sie noch "
                "anstehen.\nPrüfen Sie den Status der Anlage."
            )

    def get_drehzahl(self):
        u"""Gibt die Motordrehzahl zurück."""
        return 0 if not self.rpi.io.fu_motoran.value \
            else int(self.rpi.io.fu_ist.value / 3.77)

    def evt_btn_chvalue(self, ioname, value):
        """Gemeinsame Funktion um Werte in virtuelles Device zu schreiben."""
        self.virt.io[ioname].value = value

    def evt_btn_resetmagazin(self, ioname, value):
        if value and \
                ioname == "t_magazin1" and self.virt.io.magazin1.value != 0:
            self.add_meldung("info_magazin1")
        if value and \
                ioname == "t_magazin2" and self.virt.io.magazin2.value != 0:
            self.add_meldung("info_magazin2")

        # Magazin zurücksetzen
        self.virt.io[ioname].value = value

    def evt_lstmeldung_currentitemchanged(self):
        self.ui.btnClearOne.setEnabled(
            self.ui.lstMeldung.currentItem() is not None
        )

    def evt_mberry(self, name, value):
        if value:
            self.ui.btnMberry.setPalette(self.po_btnGreen)
        else:
            self.ui.btnMberry.setPalette(self.po_btnNormal)

    def evt_modus(self, name, value):
        if value:
            self.add_meldung(name + "_an")
        else:
            self.add_meldung(name + "_aus")

    def evt_tmr(self):
        # Uhrzeit
        self.ui.lblZeit.setText(strftime("%d.%m.%Y %H:%M:%S"))

        # Verbindungsmanagement
        if self.rpi is None:
            try:
                self._config_rpi()
                self._config_ui()
            except:
                if not self.mrk_error1:
                    self.add_meldung("err_offline")
                self.mrk_error1 = True

                # Timer neu starten
                self.tmr.start(250)
                return None
        else:
            if self.rpi.ioerrors > 0:
                self.rpi.resetioerrors()
                if not self.mrk_error1:
                    self.add_meldung("err_offline")
                self.mrk_error1 = True
            else:
                if self.mrk_error1:
                    self.add_meldung("info_online")
                self.mrk_error1 = False

        # Füllstände
        self.ui.pbMag1.setValue(self.virt.io.magazin1.value)
        self.ui.pbMag2.setValue(self.virt.io.magazin2.value)
        self.ui.gbHand.setEnabled(self.virt.io.aktivhand.value)

        # MaxFüllung
        if self.mrk_timer2 and self.virt.io.warn_mag1full.value:
            self.ui.btnResetMag1.setPalette(self.po_btnRed)
        else:
            self.ui.btnResetMag1.setPalette(self.po_btnNormal)

        if self.mrk_timer2 and self.virt.io.warn_mag2full.value:
            self.ui.btnResetMag2.setPalette(self.po_btnRed)
        else:
            self.ui.btnResetMag2.setPalette(self.po_btnNormal)

        # Anzeigen
        self.ui.lcdTemp.display(int(self.rpi.io.tmp_schrank.value / 10))
        self.ui.lcdDrehzahl.display(self.get_drehzahl())

        # Umschalter
        if self.mrk_timer1:
            self.mrk_timer2 = not self.mrk_timer2
        self.mrk_timer1 = not self.mrk_timer1

        # Timer neu starten
        self.tmr.start(250)

    def evt_setdrehzahl(self, value):
        u"""Setzt die Drehzahl für Handbetrieb."""
        self.virt.io.p_drehzahl.value = value
        self.ui.lcdHandDrehzahl.display(int(value / 3.77))

    def evt_trigger_meldungen(self, ioname, iovalue):
        # Masterfehler behoben
        if ioname == "err_master" and not iovalue:
            self.add_meldung("info_stoerreset")

        # Meldungen auswerten
        lst_meldung = self.dict_meldungen[ioname]
        if iovalue == lst_meldung[2]:
            self.add_meldung(ioname)

    def evt_valuechanged(self):
        self.virt.io.magazin1_max.value = self.ui.hsMag1.value()
        self.virt.io.magazin2_max.value = self.ui.hsMag2.value()


if __name__ == "__main__":
    app = QtWidgets.QApplication(sys.argv)
    myapp = MyApp()
    myapp.show()
    ec = app.exec_()
    if myapp.rpi is not None:
        myapp.virt.io.p_online.value = False
        myapp.rpi.cleanup()
        myapp.virt.cleanup()
    sys.exit(ec)
